Make some RERR_* choices better, and another noop_io_until_death() tweak.
[rsync/rsync.git] / cleanup.c
index 76f44e2..63118e6 100644 (file)
--- a/cleanup.c
+++ b/cleanup.c
@@ -4,11 +4,12 @@
  * Copyright (C) 1996-2000 Andrew Tridgell
  * Copyright (C) 1996 Paul Mackerras
  * Copyright (C) 2002 Martin Pool
- * Copyright (C) 2003-2007 Wayne Davison
+ * Copyright (C) 2003-2009 Wayne Davison
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License version 3 as
- * published by the Free Software Foundation.
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
 
 extern int am_server;
 extern int am_daemon;
+extern int am_sender;
+extern int am_generator;
 extern int io_error;
 extern int keep_partial;
-extern int log_got_error;
+extern int got_xfer_error;
+extern int protocol_version;
+extern int output_needs_newline;
 extern char *partial_dir;
 extern char *logfile_name;
 
@@ -77,8 +82,8 @@ void close_all(void)
  **/
 int cleanup_got_literal = 0;
 
-static char *cleanup_fname;
-static char *cleanup_new_fname;
+static const char *cleanup_fname;
+static const char *cleanup_new_fname;
 static struct file_struct *cleanup_file;
 static int cleanup_fd_r, cleanup_fd_w;
 static pid_t cleanup_pid = 0;
@@ -93,14 +98,23 @@ pid_t cleanup_child_pid = -1;
 NORETURN void _exit_cleanup(int code, const char *file, int line)
 {
        static int cleanup_step = 0;
-       static int exit_code = 0;
+       static int exit_code = 0, exit_line = 0;
+       static const char *exit_file = NULL;
        static int unmodified_code = 0;
 
        SIGACTION(SIGUSR1, SIG_IGN);
        SIGACTION(SIGUSR2, SIG_IGN);
 
-       if (exit_code) /* Preserve first error code when recursing. */
+       if (exit_code) { /* Preserve first exit info when recursing. */
                code = exit_code;
+               file = exit_file;
+               line = exit_line;
+       }
+
+       /* If this is the exit at the end of the run, the server side
+        * should not attempt to output a message (see log_exit()). */
+       if (am_server && code == 0)
+               am_server = 2;
 
        /* Some of our actions might cause a recursive call back here, so we
         * keep track of where we are in the cleanup and never repeat a step. */
@@ -108,8 +122,15 @@ NORETURN void _exit_cleanup(int code, const char *file, int line)
 #include "case_N.h" /* case 0: cleanup_step++; */
 
                exit_code = unmodified_code = code;
+               exit_file = file;
+               exit_line = line;
 
-               if (verbose > 3) {
+               if (output_needs_newline) {
+                       fputc('\n', stdout);
+                       output_needs_newline = 0;
+               }
+
+               if (DEBUG_GTE(EXIT, 2)) {
                        rprintf(FINFO,
                                "_exit_cleanup(code=%d, file=%s, line=%d): entered\n",
                                code, file, line);
@@ -133,7 +154,7 @@ NORETURN void _exit_cleanup(int code, const char *file, int line)
 
                if (cleanup_got_literal && cleanup_fname && cleanup_new_fname
                 && keep_partial && handle_partial_dir(cleanup_new_fname, PDIR_CREATE)) {
-                       char *fname = cleanup_fname;
+                       const char *fname = cleanup_fname;
                        cleanup_fname = NULL;
                        if (cleanup_fd_r != -1)
                                close(cleanup_fd_r);
@@ -148,7 +169,8 @@ NORETURN void _exit_cleanup(int code, const char *file, int line)
                /* FALLTHROUGH */
 #include "case_N.h"
 
-               io_flush(FULL_FLUSH);
+               if (!code || am_server || (!am_sender && !am_generator))
+                       io_flush(FULL_FLUSH);
 
                /* FALLTHROUGH */
 #include "case_N.h"
@@ -168,23 +190,36 @@ NORETURN void _exit_cleanup(int code, const char *file, int line)
                                code = exit_code = RERR_DEL_LIMIT;
                        if (io_error & IOERR_VANISHED)
                                code = exit_code = RERR_VANISHED;
-                       if (io_error & IOERR_GENERAL || log_got_error)
+                       if (io_error & IOERR_GENERAL || got_xfer_error)
                                code = exit_code = RERR_PARTIAL;
                }
 
-               if (code || am_daemon || (logfile_name && (am_server || !verbose)))
+               /* If line < 0, this exit is after a MSG_ERROR_EXIT event, so
+                * we don't want to output a duplicate error. */
+               if ((code && line > 0)
+                || am_daemon || (logfile_name && (am_server || !INFO_GTE(STATS, 1))))
                        log_exit(code, file, line);
 
                /* FALLTHROUGH */
 #include "case_N.h"
 
-               if (verbose > 2) {
+               if (DEBUG_GTE(EXIT, 1)) {
                        rprintf(FINFO,
                                "_exit_cleanup(code=%d, file=%s, line=%d): "
                                "about to call exit(%d)\n",
                                unmodified_code, file, line, code);
                }
 
+               /* FALLTHROUGH */
+#include "case_N.h"
+
+               if (exit_code && exit_code != RERR_SOCKETIO && exit_code != RERR_STREAMIO && exit_code != RERR_SIGNAL1
+                && (protocol_version >= 31 || (!am_sender && !am_generator))) {
+                       if (line > 0)
+                               send_msg_int(MSG_ERROR_EXIT, exit_code);
+                       noop_io_until_death();
+               }
+
                /* FALLTHROUGH */
 #include "case_N.h"
 
@@ -207,7 +242,7 @@ void cleanup_disable(void)
 }
 
 
-void cleanup_set(char *fnametmp, char *fname, struct file_struct *file,
+void cleanup_set(const char *fnametmp, const char *fname, struct file_struct *file,
                 int fd_r, int fd_w)
 {
        cleanup_fname = fnametmp;